// 																																			KEDS Project
// 																												(Kansas Event Data System)
//__________________________________________________________________________________
// 																																	Count_Types.c 

// This program goes through an data set counts the verbal/material cooperation/
// conflict WEIS event types on a dyadic basis.  The dyad to be compiled and the
// input data file are set through #defines

// INPUT FILE
// IN_FILE -- Standard KEDS/TABARI WEIS event data format: see 
//               http://www.ukans.edu/~keds/data.html
//            for details.
//
// OUTPUT FILE
// Text file containing the counts as a tab-delimited file with the actor codes 
// as prefix (in dyad order) and the labels.  File name is constructed from the
// actor IDs.
//
//		VERCP -- verbal cooperation
//    MATCP -- material cooperation
//		VERCF -- verbal conflict
//    MATCF -- material conflict
//

//	Programmer: Philip A. Schrodt
//	Compiler:		Metrowerks C
//	Language:		ANSI C except for // comments

//	Revision History: 
//		01.07		Original program Count_Events -- Modified from "SurvivorStat" program
//    03.02   This program modified from Count_Events, mostly removing code 
//    05.01   Modified to produce weekly data -- see comments in code 

//__________________________________________________________________________________

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//__________________________________________________________________________________
// 	Macros

#define	FALSE			0
#define TRUE			1

#define EXCEL_OFFSET  20454  // convert from Excel to Stata Julian dates

#define MAX_WEIS  4			// number of WEIS aggregations
#define MAX_KOUNT 8			// number of totalled variables

#define IN_FILE		"LEVANT.WEIS"					// input data file
#define OUT_FILE_SUFFIX		"COUNT.TYPES.OUT"
#define ANTAGO1_LIST "ISR."  					// codes for the actors we are tabulating,
#define ANTAGO2_LIST "LEB."  					// separating by periods to avoid accidental matches

																			// Julian bounds for the data set
#define	HIGH_DAY		36890 - EXCEL_OFFSET 		//	04.12.31
#define	LOW_DAY   	27497 - EXCEL_OFFSET		//	79.04.14


//___________________________________________________________________________
// Prototypes

void Initialize(void);      // initialize globals
long JulianStata (char *s);	// Converts dates of form YYMMDD to Julian serial beginning 0 = 1 Jan 1960
long CheckDate(void); 			// returns Julian date if date is okay, otherwise zero
void WriteRecord(void); 		// write the resulting record
void GetIndicators(void);  	// accumulate data on independent variables
void ResetIndicators(void); // re-initialize kount[] 


//__________________________________________________________
// Globals

int		Month[] = {0,0,31,59,90,120,151,181,212,243,273,304,334};		// used for Julian computation: # days prior to Month[n] JAN=1
char  WEIS_Catg[MAX_WEIS][24]; // holds WEIS aggregations
char  WEIS_Type[MAX_WEIS][6];  // labels for WEIS aggregations
int		kount[MAX_KOUNT];			// frequency variables
char  outfile_name[32];
int   nrec = 0;             // record count
									
char  santago1[5];  	// codes for antagonist group 1
char  santago2[5];		// codes for antagonist group 2
long	high_day, low_day;

char  src[7];			// source and target codes 
char  tar[7];
char  code[4];		// event code 
char  date[7];		// character version of date of current record
char	stail[256];	// read junk from end of input line
char  curdate[7] = "000000";		// character version of date of record that set previous state change

//char  prevdate[7];		// character version of date of current record

FILE	*fin, *fout; // input and output files

//__________________________________________________________________________________
//																																			main program

void main(void)
{
int 	ka;
long	ijul;
long oldjul = 0;

	Initialize();

	fin = fopen(IN_FILE,"r");
	if (!fin) { printf("Error: can't open %s",IN_FILE); exit(0); }

	strcpy(outfile_name,ANTAGO1_LIST);  // assemble output file name
	strcat(outfile_name,ANTAGO2_LIST);
	strcat(outfile_name,OUT_FILE_SUFFIX);
	fout = fopen(outfile_name,"w");
	if (!fout) { printf("Error: can't open %s",outfile_name); exit(0); }
	// write variable labels
	fprintf(fout,"record\tdate");
	for (ka=0; ka<MAX_WEIS; ++ka) fprintf(fout,"\t%s%s%s",ANTAGO1_LIST,ANTAGO2_LIST,WEIS_Type[ka]);
	for (ka=0; ka<MAX_WEIS; ++ka) fprintf(fout,"\t%s%s%s",ANTAGO2_LIST,ANTAGO1_LIST,WEIS_Type[ka]);
	fprintf(fout,"\n");

	while (!feof(fin))
	{
		fscanf(fin,"%s %s %s %s",&date, &src, &tar, &code);
		fgets(stail,256,fin);

#if FALSE  // this code block does monthly data
		if (date[3] != curdate[3]) {		// output when we hit a new month
			WriteRecord(); 		// write the resulting record
			strcpy(curdate,date);
			ResetIndicators();  // initialize indicators array
		}
		
		ijul = CheckDate();
		if (!ijul) continue;		// escape bad dates 
#endif

		ijul = CheckDate();
		if ((ijul != oldjul) && (2 == ijul % 7 )) {		// output when we hit a new Sunday
			WriteRecord(); 		// write the resulting record
			strcpy(curdate,date);
			oldjul = ijul;
			ResetIndicators();  // initialize indicators array
		}
		
		if (!ijul) continue;		// escape bad dates 
		GetIndicators();
	}
	
	fclose(fin);
	WriteRecord(); 		// write the final record

	fclose(fout);

} // main

void Initialize(void)
// initialize globals
{
// WEIS aggregations
	strcpy(WEIS_Catg[0],"02.03.04.05.08.09.10");
	strcpy(WEIS_Catg[1],"01.06.07");
	strcpy(WEIS_Catg[2],"11.12.13.14.15.16.17");
	strcpy(WEIS_Catg[3],"18.19.20.21.22");
// WEIS aggregation labels
	strcpy(WEIS_Type[0],"VERCP");
	strcpy(WEIS_Type[1],"MATCP");
	strcpy(WEIS_Type[2],"VERCF");
	strcpy(WEIS_Type[3],"MATCF");
	
	strcpy(santago1,ANTAGO1_LIST);
	strcpy(santago2,ANTAGO2_LIST);
		
} // Initialize


long JulianStata (char *s)
// Converts dates of form YYMMDD to Julian serial beginning 0 = 1 Jan 1960
// (which matches the Stata convention) 
{
int 	mo,dy;
long	yr, ij;

  for (mo=0; mo<7; mo++) if (s[mo] == ' ') s[mo] = '0';			// convert blanks to zeros
  
																		//Break date out of the string and convert to integers 
	yr = (short)s[0] * 10 + (short)s[1] - 588;			// 588 substracts the 1960 base from yr
	if (yr<0) yr +=100;
	mo = (short)s[2] * 10 + (short)s[3] - 528;
	dy = (short)s[4] * 10 + (short)s[5] - 528;
						
	ij =  yr * 365;
	ij += ((yr+3) / 4) + Month[mo] + (dy-1);				// ((yr+3)/4) is the leap-year adjustment
	if (((yr % 4) == 0) && (mo > 2)) ij++;	// adjust for month within a leap-year
	return ij;
} // JulianStata

long CheckDate(void)
// check this because we don't trust Reuters;
// function returns Julian date if date is okay, otherwise zero
{
long	ijul = JulianStata(date);
//char c;
//char *pc;
//int ka;


	if ((ijul<LOW_DAY) || (ijul>HIGH_DAY))	{				
		printf("Date error:\"%s\"  %d\n",date,ijul);
#if FALSE
		printf("Previous: %s",prevdate);
		printf("Codes: %s %s %s",src,tar,code);
		pc = date;
		while (*pc) { printf("%d - ",(int)*pc); ++pc;}
		printf("\n");
		c = getchar(); // wait for return
#endif
		return 0;	
		}
		else {
//		 strcpy(prevdate,date);
		 return ijul;
		}
} // CheckDate


void WriteRecord(void) 
// write the month record
{
int ka;

	if ('0' == curdate[1]) return; // don't write initialization record 
	++nrec;                       // increment record number
//	fprintf(fout,"%5d\t%2.2s-%2.2s",nrec, curdate,&curdate[2]);  // output for monthly data
//	printf("%2.2s-%2.2s\n",curdate,&curdate[2]);
	fprintf(fout,"%5d\t%2.2s-%2.2s-%2.2s",nrec, curdate,&curdate[2],&curdate[4]); // output for weekly data
	printf("%2.2s-%2.2s-%2.2s\n",curdate,&curdate[2],&curdate[4]);
	for (ka=0; ka<MAX_KOUNT; ++ka) fprintf(fout,"\t%5d",kount[ka]); // counts
	fprintf(fout,"\n");
} // WriteRecord

void GetIndicators(void) 
// accumulate the information on the independent variables
{
char cuecode[3];

	cuecode[0] = code[0];
	cuecode[1] = code[1];
	cuecode[2] = '\0';
	
	if (strstr(santago1,src) && strstr(santago2,tar)) {  // behavior from A to B
			if (strstr(WEIS_Catg[0],cuecode)) ++kount[0]; 	// "VERCPA2B"
			if (strstr(WEIS_Catg[1],cuecode)) ++kount[1]; 	// "MATCPA2B"
			if (strstr(WEIS_Catg[2],cuecode)) ++kount[2]; 	// "VERCFA2B"
			if (strstr(WEIS_Catg[3],cuecode)) ++kount[3]; 	// "MATCFA2B"
	}
	else 
	if (strstr(santago1,tar) &&  strstr(santago2,src)) { // behavior from B to A
			if (strstr(WEIS_Catg[0],cuecode)) ++kount[4]; 	// "VERCPB2A"
			if (strstr(WEIS_Catg[1],cuecode)) ++kount[5]; 	// "MATCPB2A"
			if (strstr(WEIS_Catg[2],cuecode)) ++kount[6]; 	// "VERCFB2A"
			if (strstr(WEIS_Catg[3],cuecode)) ++kount[7]; 	// "MATCFB2A"
	}

} // GetIndicators

void ResetIndicators(void)
// re-initialize kount[] 
{
int ka;
	for (ka=0; ka<MAX_KOUNT; ++ka) kount[ka] = 0;
}
// ResetIndicators

